@@ -1,7 +1,7 @@
 /*
  * Miscellaneous routines for Mathomatic.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -343,7 +343,7 @@ int		n;		/* expression length */
 }
 
 /*
- * This is called when the max expression size has been exceeded.
+ * This is called when the maximum expression size has been exceeded.
  *
  * There is no return.
  */
@@ -368,7 +368,7 @@ char	*cp;
 		i = cur_equation;
 	} else {
 		i = decstrtol(cp, &cp) - 1;
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return -1;
 	}
 	if (not_defined(i)) {
@@ -419,6 +419,8 @@ char		*cp;		/* string to parse */
 			blt(equation, lhs[i], n_lhs[i] * sizeof(token_type));
 			*np = n_lhs[i];
 		}
+		list_proc(equation, *np, false);
+		fprintf(gfp, "\n");
 		return cp2;
 	}
 	return parse_section(equation, np, cp);
@@ -446,7 +448,7 @@ int		*np;		/* pointer to the returned parsed expression length */
 			str_tolower(cp);
 		}
 		cp = parse_expr(equation, np, cp);
-		if (cp && !extra_garbage(cp)) {
+		if (cp && !extra_characters(cp)) {
 			break;
 		}
 	}
@@ -474,7 +476,7 @@ long	*vp;	/* pointer to the returned variable */
 			return false;
 		}
 		cp = parse_var2(vp, cp);
-		if (cp == NULL || extra_garbage(cp)) {
+		if (cp == NULL || extra_characters(cp)) {
 			continue;
 		}
 		return true;
@@ -610,6 +612,7 @@ int	en;	/* equation space number */
 		if (factor_int_flag) {
 			factor_int_sub(en);
 		}
+		free(result_str);
 		result_str = list_equation(en, false);
 		return(result_str != NULL);
 	}
@@ -710,11 +713,11 @@ int	*ip, *jp;
  * on a command line.
  *
  * Returns false if OK.
- * Returns true if non-space characters were encountered
+ * Returns true if any non-space characters were encountered
  * and an error message was printed.
  */
 int
-extra_garbage(cp)
+extra_characters(cp)
 char	*cp;	/* command line */
 {
 	if (cp) {
@@ -738,7 +741,7 @@ int	*ip, *jp;
 	if (!get_range(cpp, ip, jp)) {
 		return false;
 	}
-	if (extra_garbage(*cpp)) {
+	if (extra_characters(*cpp)) {
 		return false;
 	}
 	return true;
