@@ -1,7 +1,7 @@
 /*
  * Mathomatic commands that don't belong anywhere else.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -28,18 +28,15 @@ int
 version_cmd(cp)
 char	*cp;	/* the command line argument */
 {
-	if (extra_garbage(cp)) {
-		/* if anything on the command line, fail */
-		return false;
-	}
 #if	LIBRARY
+	free(result_str);
 	result_str = strdup(VERSION);
 #endif
 	return version_report();
 }
 
 /*
- * Report version number and compile flags.
+ * Report version number, compile flags, and memory usage.
  */
 int
 version_report()
@@ -72,7 +69,7 @@ version_report()
 #if	I18N
 	fprintf(gfp, "I18N ");
 #endif
-	fprintf(gfp, "\n");
+	fprintf(gfp, _("\nMaximum memory usage: %lld kilobytes\n"), (long long) (N_EQUATIONS + 3LL) * n_tokens * sizeof(token_type) * 2LL / 1000LL);
 	return true;
 }
 
@@ -104,7 +101,7 @@ char	*cp;
 	if ((cp = parse_equation(i, cp)) != NULL) {
 		if (n_lhs[i] && n_rhs[i]) {
 			error(_("Invalid argument."));
-		} else if (!extra_garbage(cp) && solve(i, cur_equation)) {
+		} else if (!extra_characters(cp) && solve(i, cur_equation)) {
 			return return_result(cur_equation);
 		}
 	}
@@ -224,7 +221,7 @@ int	product_flag;	/* true for product, otherwise sum */
 			return false;
 		}
 	}
-	if (extra_garbage(cp))
+	if (extra_characters(cp))
 		return false;
 	count_down = (end < start);
 	if (fmod(fabs(start - end) / step, 1.0)) {
@@ -517,7 +514,10 @@ char	*cp;
 		return false;
 	}
 	for (; i <= j; i++) {
-		push_en(i);
+		if (!push_en(i)) {
+			error(_("Push failed."));
+			return false;
+		}
 	}
 	debug_string(0, _("Expression(s) pushed.  Press the UP key to access."));
 	return true;
@@ -525,13 +525,21 @@ char	*cp;
 
 /*
  * Push an equation space into the readline history.
+ *
+ * Return true if successful.
  */
+int
 push_en(en)
 int	en;	/* equation space number */
 {
-	high_prec = true;
-	add_history(list_equation(en, false));
-	high_prec = false;
+	if (readline_enabled) {
+		high_prec = true;
+		add_history(list_equation(en, false));
+		high_prec = false;
+		return true;
+	} else {
+		return false;
+	}
 }
 #endif
 
@@ -689,11 +697,15 @@ next_option:
 #endif
 	if (strncasecmp(cp, "special", 7) == 0) {
 		cp = skip_param(cp);
-		if (negate || *cp == '\0' || cp[1]) {
-			error(_("Please specify a single character."));
-			return false;
+		if (negate) {
+			special_variable_character = '\0';
+		} else {
+			if (*cp == '\0' || cp[1]) {
+				error(_("Please specify a single character."));
+				return false;
+			}
+			special_variable_character = *cp;
 		}
-		special_variable_character = *cp;
 		return true;
 	}
 	if (strncasecmp(cp, "columns", 7) == 0) {
@@ -766,6 +778,17 @@ next_option:
 	return false;
 }
 
+/*
+ * The echo command.
+ */
+int
+echo_cmd(cp)
+char	*cp;
+{
+	fprintf(gfp, "%s\n", cp);
+	return true;
+}
+
 /*
  * The pause command.
  */
@@ -777,7 +800,7 @@ char	*cp;
 	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
-		cp = _("Please Press Enter");
+		cp = _("Please press Enter");
 	}
 	snprintf(prompt_str, sizeof(prompt_str), " ==== %s ==== ", cp);
 	if ((cp1 = get_string(buf, sizeof(buf))) == NULL) {
@@ -786,6 +809,9 @@ char	*cp;
 	if (strncasecmp(cp1, "quit", 4) == 0) {
 		return false;
 	}
+	if (strncasecmp(cp1, "exit", 4) == 0) {
+		return false;
+	}
 	return true;
 }
 
@@ -859,7 +885,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 		if (cp == NULL) {
 			return false;
 		}
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return false;
 	}
 	simp_loop(source, nps);
@@ -939,7 +965,7 @@ char	*cp;
 	int	i;
 	double	count;
 
-	if (extra_garbage(cp))
+	if (extra_characters(cp))
 		return false;
 	trhs[0] = zero_token;
 	n_trhs = 1;
@@ -1289,7 +1315,7 @@ char	*cp;
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
-			fprintf(gfp, _("Expressions may differ.\n"));
+			fprintf(gfp, _("Expressions differ.\n"));
 			return false;
 		}
 		error(_("Cannot compare an expression with an equation."));
@@ -1350,7 +1376,7 @@ char	*cp;
 		fprintf(gfp, _("Equations are identical.\n"));
 		return true;
 	}
-	fprintf(gfp, _("Equations may differ.\n"));
+	fprintf(gfp, _("Equations differ.\n"));
 	return false;
 times_neg1:
 	if (!diff_sign && lhs[i][0].token.variable == lhs[j][0].token.variable) {
@@ -1388,7 +1414,7 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return false;
 	}
 	i = next_espace();
@@ -1405,8 +1431,8 @@ char	*cp;
 	calc_simp(rhs[i], &nr);
 	if (get_constant(rhs[i], nr, &d1) && get_constant(lhs[i], nl, &d2)) {
 		d4 = modf(d1 / d2, &d3);
-		fprintf(gfp, _("Result of numerical division: %.14g\n"), d1 / d2);
-		fprintf(gfp, _("Quotient: %.14g, Remainder: %.14g\n"), d3, d4 * d2);
+		fprintf(gfp, _("Result of numerical division: %.*g/%.*g = %.*g\n"), precision, d1, precision, d2, precision, d1 / d2);
+		fprintf(gfp, _("Quotient: %.*g, Remainder: %.*g\n"), precision, d3, precision, d4 * d2);
 		d1 = fabs(d1);
 		d2 = fabs(d2);
 		d3 = gcd(d1, d2);
@@ -1414,14 +1440,14 @@ char	*cp;
 			fprintf(gfp, _("No GCD found.\n"));
 			return true;
 		}
-		fprintf(gfp, _("Greatest Common Divisor (GCD) = %.14g\n"), d3);
-		fprintf(gfp, _("Least Common Multiple (LCM) = %.14g\n"), (d1 * d2) / d3);
+		fprintf(gfp, _("Greatest Common Divisor (GCD) = %.*g\n"), precision, d3);
+		fprintf(gfp, _("Least Common Multiple (LCM) = %.*g\n"), precision, (d1 * d2) / d3);
 		return true;
 	}
 	if (parse_complex(rhs[i], nr, &c1) && parse_complex(lhs[i], nl, &c2)) {
 		c3 = complex_div(c1, c2);
-		fprintf(gfp, _("Result of complex number division: "));
-		fprintf(gfp, "%.*g%+.*g*i#\n\n", precision, c3.re, precision, c3.im);
+		fprintf(gfp, _("Result of complex number division:\n"));
+		fprintf(gfp, "%.*g %+.*g*i\n\n", precision, c3.re, precision, c3.im);
 		return true;
 	}
 	v_tmp = v;
@@ -1445,7 +1471,7 @@ char	*cp;
 	}
 	if (j) {
 		simp_divide(trhs, &n_trhs);
-		fprintf(gfp, _("Polynomial Greatest Common Divisor (iterations = %d):\n"), j);
+		fprintf(gfp, _("Polynomial GCD (Euclidean algorithm iterations = %d):\n"), j);
 		list_proc(trhs, n_trhs, false);
 		fprintf(gfp, "\n");
 	} else {
@@ -1598,12 +1624,12 @@ long	v;
 }
 
 /*
- * The flist or display command.
+ * The display command.
  *
- * Displays equations in fraction format.
+ * Displays equations in multi-line fraction format.
  */
 int
-flist_cmd(cp)
+display_cmd(cp)
 char	*cp;
 {
 	int	i, j;
@@ -1624,6 +1650,7 @@ char	*cp;
 			}
 #if	LIBRARY
 			if (gfp == stdout) {
+				free(result_str);
 				result_str = list_equation(i, false);
 				return(result_str != NULL);
 			}
@@ -1661,6 +1688,7 @@ char	*cp;
 		}
 #if	LIBRARY
 		if (gfp == stdout) {
+			free(result_str);
 			result_str = list_equation(i, export_flag);
 			return(result_str != NULL);
 		}
@@ -1747,7 +1775,7 @@ char	*cp;
 	i = cur_equation;
 	if (strcmp_tospace(cp, "constants") == 0) {
 		cp = skip_param(cp);
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return false;
 		subst_constants(lhs[i], &n_lhs[i]);
 		subst_constants(rhs[i], &n_rhs[i]);
@@ -1821,7 +1849,7 @@ char	*cp;
 				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL || n <= 0) {
 					return false;
 				}
-				if (extra_garbage(cp1))
+				if (extra_characters(cp1))
 					return false;
 				goto do_this;
 			}
@@ -1915,7 +1943,7 @@ char	*cp;
 			d = strtod(cp, &cp);
 			cp = skip_space(cp);
 			if (!factor_one(d)) {
-				error(_("Not a valid integer."));
+				error(_("Not a valid integer or number too large."));
 				return false;
 			}
 			display_unique();
@@ -2004,14 +2032,14 @@ int
 quit_cmd(cp)
 char	*cp;
 {
-	if (extra_garbage(cp))
+	if (extra_characters(cp))
 		return false;
 	exit_program(0);
 	return false;
 }
 #endif
 
-#if	!SECURE && !LIBRARY
+#if	!SECURE
 /*
  * The read command.
  */
@@ -2021,7 +2049,6 @@ char	*cp;
 {
 	int	rv;
 	FILE	*fp;
-	jmp_buf	save_save;
 	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
@@ -2034,31 +2061,54 @@ char	*cp;
 	if (fp == NULL) {
 		fp = fopen(cp, "r");
 		if (fp == NULL) {
-			error(_("Can't open file."));
+			error(_("Can't open input file."));
 			return false;
 		}
+	} else {
+		cp = buf;
 	}
+	rv = read_sub(fp);
+	if (rv) {
+		printf(_("Read operation aborted.\n"));
+	} else {
+		printf(_("Finished reading file \"%s\".\n"), cp);
+	}
+	fclose(fp);
+	return(!rv);
+}
+
+/*
+ * Read and process Mathomatic input from a file pointer.
+ *
+ * Return zero if no error, non-zero if aborted.
+ */
+int
+read_sub(fp)
+FILE	*fp;
+{
+	int	rv;
+	jmp_buf	save_save;
+	char	*cp;
+
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if ((rv = setjmp(jmp_save)) != 0) {
 		clean_up();
-		if (rv == 14)
+		if (rv == 14) {
 			error(_("Expression too large."));
-		printf(_("Read operation aborted.\n"));
-		goto end_read;
-	}
-	while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
-		default_color();
-		input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
-		printf("%s", cp);
-		set_error_level(cp);
-		if (!process(cp)) {
-			longjmp(jmp_save, 3);
+		}
+	} else {
+		while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
+			default_color();
+			input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
+			printf("%s", cp);
+			set_error_level(cp);
+			if (!process(cp)) {
+				longjmp(jmp_save, 3);
+			}
 		}
 	}
-end_read:
 	blt(jmp_save, save_save, sizeof(jmp_save));
-	fclose(fp);
-	return(!rv);
+	return rv;
 }
 #endif
 
@@ -2077,7 +2127,7 @@ char	*cp;
 
 	if (*cp == '\0') {
 #if	CYGWIN
-		my_strlcpy(tmp_file, "mathxxx.tmp", sizeof(tmp_file));
+		my_strlcpy(tmp_file, "math.tmp", sizeof(tmp_file));
 		fp = fopen(tmp_file, "w+");
 		if (fp == NULL) {
 			error(_("Can't create temporary file."));
@@ -2136,7 +2186,7 @@ edit_again:
 	}
 	clear_all();
 	if (!read_cmd(cp)) {
-		if (pause_cmd("Prepare to run the editor")) {
+		if (pause_cmd(_("Prepare to run the editor"))) {
 			goto edit_again;
 		}
 	}
