@@ -1,7 +1,7 @@
 /*
  * Mathomatic differentiation routines and commands.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -287,8 +287,8 @@ int
 derivative_cmd(cp)
 char	*cp;
 {
-	long		v = 0;
 	int		i;
+	long		v = 0;
 	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
@@ -379,6 +379,7 @@ char	*cp;
 {
 	int		i;
 	long		v = 0;
+	long		l1, order = 1;
 	token_type	want;
 	token_type	*source;
 	int		n;
@@ -398,11 +399,17 @@ char	*cp;
 		n = n_lhs[cur_equation];
 	}
 	if (*cp) {
-		cp = parse_var2(&v, cp);
-		if (cp == NULL) {
-			return false;
+		if (!isdigit(*cp)) {
+			cp = parse_var2(&v, cp);
+			if (cp == NULL) {
+				return false;
+			}
 		}
-		if (extra_garbage(cp)) {
+		if (*cp) {
+			order = decstrtol(cp, &cp);
+		}
+		if (*cp || order <= 0) {
+			error(_("The order must be a positive integer."));
 			return false;
 		}
 	}
@@ -421,14 +428,16 @@ char	*cp;
 	}
 	i = next_espace();
 	blt(rhs[i], source, n * sizeof(token_type));
-/* take the derivative with respect to the specified variable and simplify: */
-	if (!differentiate(rhs[i], &n, v)) {
-		error(_("Differentiation failed."));
-		return false;
+/* take derivatives with respect to the specified variable and simplify: */
+	for (l1 = 0; l1 < order; l1++) {
+		if (!differentiate(rhs[i], &n, v)) {
+			error(_("Differentiation failed."));
+			return false;
+		}
+		simpa_side(rhs[i], &n, true);
 	}
-	simpa_side(rhs[i], &n, true);
 	if (!found_var(rhs[i], n, v)) {
-		error(_("There are no extrema for this equation."));
+		error(_("There are no solutions."));
 		return false;
 	}
 	n_rhs[i] = n;
@@ -532,20 +541,16 @@ char	*cp;
 		if ((cp = parse_section(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
 			return false;
 		}
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return false;
 	} else {
 #if	!SILENT
 		list_var(v, 0);
-		printf(_("Taylor series expansion about (%s) = point.\n"), var_str);
+		printf(_("Taylor series expansion about %s = point.\n"), var_str);
 #endif
-		for (;;) {
-			my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
-			if (!get_expr(lhs[our], &our_nlhs)) {
-				error(_("Enter \"0\" if you don't know."));
-				continue;
-			}
-			break;
+		my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
+		if (!get_expr(lhs[our], &our_nlhs)) {
+			return false;
 		}
 	}
 	if (order < 0) {
@@ -652,6 +657,9 @@ loop_again:
 			goto loop_again;
 		}
 	}
+#if	!SILENT
+	printf(_("%ld derivative%s applied.\n"), n, (n == 1) ? "" : "s");
+#endif
 	if (n_rhs[cur_equation]) {
 		n_lhs[i] = n_lhs[cur_equation];
 	}
@@ -727,7 +735,7 @@ char	*cp;
 			return false;
 		}
 	}
-	if (extra_garbage(cp))
+	if (extra_characters(cp))
 		return false;
 /* copy to a new equation space and work on the copy: */
 	i = next_espace();
@@ -736,7 +744,7 @@ char	*cp;
 /* see if the limit expression contains infinity: */
 	simp_loop(tes, &n_tes);
 	infinity_flag = exp_contains_infinity(tes, n_tes);
-/* fully simplify the current equation and group denominators together: */
+/* first fully simplify the current equation and group denominators together: */
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
 	found_den = complex_den = false;
@@ -777,12 +785,13 @@ char	*cp;
 			break;
 		}
 	}
-	if (!found_den || (infinity_flag && !complex_den)) {
+	if (infinity_flag ? (!complex_den && found_den) : (!found_den)) {
 /* in this case, just substitute the limit variable with the limit expression and simplify: */
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tes, n_tes, v);
 		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 		return return_result(cur_equation);
 	}
+	debug_string(1, "Taking the limit by solving...");
 	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == HUGE_VAL) {
 /* To take the limit to positive infinity, */
 /* replace infinity with zero and replace the limit variable with its reciprocal: */
