@@ -1,7 +1,7 @@
 /*
- * Mathomatic constant factorizing routines.
+ * Mathomatic floating point constant factorizing routines.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -9,8 +9,9 @@
 static void	try_factor();
 static int	fc_recurse();
 
+/* The following data is used to factor integers: */
 static double nn, vv;
-static double sq[] = {		/* Additive array that skips over multiples of 2, 3, 5, and 7. */
+static double skip_multiples[] = {	/* Additive array that skips over multiples of 2, 3, 5, and 7. */
 	10, 2, 4, 2, 4, 6, 2, 6,
 	 4, 2, 4, 6, 6, 2, 6, 4,
 	 2, 6, 4, 6, 8, 4, 2, 4,
@@ -30,7 +31,7 @@ factor_one(start)
 double	start;
 {
 	int	i;
-	double	ii;
+	double	d;
 
 	uno = 0;
 	nn = start;
@@ -50,18 +51,18 @@ double	start;
 	try_factor(3.0);
 	try_factor(5.0);
 	try_factor(7.0);
-	ii = 1.0;
-	while (ii <= vv) {
-		for (i = 0; i < ARR_CNT(sq); i++) {
-			ii += sq[i];
-			try_factor(ii);
+	d = 1.0;
+	while (d <= vv) {
+		for (i = 0; i < ARR_CNT(skip_multiples); i++) {
+			d += skip_multiples[i];
+			try_factor(d);
 		}
 	}
 	if (nn != 1.0) {
 		try_factor(nn);
 	}
 	if (start != multiply_out_unique()) {
-		error("Error factoring integers.");
+		error("Internal error factoring integers.");
 		return false;
 	}
 	return true;
@@ -131,9 +132,9 @@ display_unique()
 }
 
 /*
- * Factor integers in an expression.
+ * Factor integers in an equation side.
  *
- * Return true if expression was modified.
+ * Return true if equation side was modified.
  */
 int
 factor_int(equation, np)
@@ -207,21 +208,25 @@ int		*np;
 factor_int_sub(n)
 int	n;	/* equation space number */
 {
+	if (n_lhs[n] <= 0)
+		return;
 	factor_int(lhs[n], &n_lhs[n]);
 	factor_int(rhs[n], &n_rhs[n]);
 }
 
 /*
- * Factor constants in equation side.
+ * Neatly factor out coefficients in additive expressions in an equation side.
+ * For example: (2*x + 4*y + 6) becomes 2*(x + 2*y + 3).
  *
  * This routine is often necessary because the expression compare (se_compare())
  * does not return a multiplier (except for +/-1.0).
  * This routine is not used during polynomial operations.
- * It is required for simplification of algebraic fractions, etc.
+ * Normalization done here is required for simplification of algebraic fractions, etc.
  *
  * If "level_code" is 0, all additive expressions are normalized
- * by making at least one coefficient unity by factoring out
- * the smallest constant.
+ * by making at least one coefficient unity (1) by factoring out
+ * the smallest constant.  The absolute value of all other coefficients will be >= 1.
+ * If all coefficients are negative, -1 will be factored out, too.
  *
  * If "level_code" is 1, any level 1 additive expression is factored
  * nicely for readability, while all deeper levels are normalized.
@@ -235,9 +240,9 @@ int	n;	/* equation space number */
  */
 int
 factor_constants(equation, np, level_code)
-token_type	*equation;
-int		*np;
-int		level_code;
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to length of equation side */
+int		level_code;	/* see above */
 {
 	if (level_code > 2)
 		return false;
@@ -283,7 +288,7 @@ break_cont:
 				if (first) {
 					minimum = d;
 					first = false;
-				} else if (d < minimum) {
+				} else if (minimum > d) {
 					minimum = d;
 				}
 				break;
@@ -304,15 +309,14 @@ break_cont:
 				if (first) {
 					minimum = 1.0;
 					first = false;
-				} else if (1.0 < minimum)
+				} else if (minimum > 1.0) {
 					minimum = 1.0;
+				}
 				break;
 			}
 		} else {
 			op = 0;
-			for (j = i + 1;; j += 2) {
-				if (j >= *np || equation[j].level <= level)
-					break;
+			for (j = i + 1; j < *np && equation[j].level > level; j += 2) {
 				if (equation[j].level == level + 1) {
 					op = equation[j].token.operatr;
 				}
