@@ -3,7 +3,7 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -35,16 +35,17 @@ static com_type com_list[] = {
 {	"compare",	NULL,			compare_cmd,	"equation-number [\"with\" equation-number]",		"Compare two equation spaces to see if they are the same." },
 {	"copy",		NULL,			copy_cmd,	"[equation-number-range]",				"Duplicate the specified equation spaces." },
 {	"derivative",	"differentiate",	derivative_cmd,	"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+{	"display",	"flist",		display_cmd,	"[\"factor\"] [equation-number-range]",			"Display stored expressions in multi-line fraction format." },
 #if	!LIBRARY
 {	"divide",	NULL,			divide_cmd,	"[variable]",						"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
-#if	(UNIX || CYGWIN) && !SECURE
+#endif
+{	"echo",		NULL,			echo_cmd,	"[text]",						"Output a line of text, followed by a newline." },
+#if	!LIBRARY && (UNIX || CYGWIN) && !SECURE
 {	"edit",		NULL,			edit_cmd,	"[file-name]",						"Edit all equation spaces or an input file." },
 #endif
-#endif
 {	"eliminate",	NULL,			eliminate_cmd,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
-{	"extrema",	"extremum",		extrema_cmd,	"[variable]",						"Find the minimums and maximums of the current expression." },
+{	"extrema",	"stationary",		extrema_cmd,	"[variable] [order]",					"Find possible local minimums and maximums of the current expression." },
 {	"factor",	NULL,			factor_cmd,	"[\"number\" [integers]] or [equation-number-range] [variables]",	"Trivially factor integers or stored expressions." },
-{	"flist",	"display",		flist_cmd,	"[\"factor\"] [equation-number-range]",			"Display stored expressions in multi-line fraction format." },
 {	"fraction",	NULL,			fraction_cmd,	"[equation-number-range]",				"Convert expressions with any algebraic fractions into a single fraction." },
 {	"help",		NULL,			help_cmd,	"[topic or command-name]",				"Short, built-in help." },
 {	"imaginary",	NULL,			imaginary_cmd,	"[variable]",						"Copy the imaginary part of an expression (see the \"real\" command)." },
@@ -66,7 +67,7 @@ static com_type com_list[] = {
 #if	!LIBRARY
 {	"quit",		"exit",			quit_cmd,	"",							"Terminate this program without saving." },
 #endif
-#if	!SECURE && !LIBRARY
+#if	!SECURE
 {	"read",		NULL,			read_cmd,	"file-name",						"Read in a text file as if it was typed in." },
 #endif
 {	"real",		NULL,			real_cmd,	"[variable]",						"Copy the real part of an expression (see the \"imaginary\" command)." },
@@ -77,7 +78,7 @@ static com_type com_list[] = {
 #if	!SECURE
 {	"save",		NULL,			save_cmd,	"file-name",						"Save all stored expressions and equations in a text file." },
 #endif
-{	"set",		NULL,			set_cmd,	"[[\"no\"] options]",					"Set or display various session options." },
+{	"set",		NULL,			set_cmd,	"[[\"no\"] option]",					"Set or display various session options." },
 {	"simplify",	NULL,			simplify_cmd,	"[\"symbolic\"] [\"quick\"] [equation-number-range]",	"Completely simplify stored expressions." },
 {	"solve",	NULL,			solve_cmd,	"variable or \"0\"",					"Solve the current equation for a variable or for 0." },
 {	"sum",		NULL,			sum_cmd,	"variable start end [step]",				"Compute the summation as variable goes from start to end." },
@@ -86,7 +87,7 @@ static com_type com_list[] = {
 #endif
 {	"taylor",	NULL,			taylor_cmd,	"[variable] [order] [point]",				"Compute the Taylor series expansion of the current expression." },
 {	"unfactor",	"expand",		unfactor_cmd,	"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) stored expressions." },
-{	"version",	NULL,			version_cmd,	"",							"Display version number and compile flags used." }
+{	"version",	NULL,			version_cmd,	"",							"Display version number, compile flags used, and maximum memory usage." }
 };
 
 /*
@@ -279,7 +280,7 @@ char	*cp;
 /* See if the string pointed to by "cp" is a command. */
 /* If so, execute it. */
 	cp1 = cp;
-	while (*cp1 && isalpha(*cp1))
+	while (*cp1 && !isspace(*cp1))
 		cp1++;
 	for (i = 0; i < ARR_CNT(com_list); i++) {
 		if (((cp1 - cp) >= min(4, strlen(com_list[i].name)) && strncasecmp(cp, com_list[i].name, cp1 - cp) == 0)
@@ -483,14 +484,15 @@ char	*cp;
 			P("    + add             - subtract\n");
 			P("Operators in the same precedence level are evaluated left to right.\n");
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
-			P("Constants always start with a digit (0..9) or a period.");
-			P("Predefined constants and variables follow:\n");
-			P("    e# - the universal constant e (2.7182818284...).");
-			P("    pi - the universal constant pi (3.1415926535...).");
-			P("    i# - imaginary number (square root of -1).");
-			P("    sign, sign1, sign2, sign3, ... - may only be +1 or -1.");
-			P("    integer - may be any integer.");
-			P("    inf - infinity constant.\n");
+			P("Constants always start with a digit (0..9) or a period (.).");
+			P("Predefined constants and variables follow:");
+			P("    e or e# - the universal constant e (2.7182818284...).");
+			P("    pi or pi# - the universal constant pi (3.1415926535...).");
+			P("    i or i# - imaginary number (square root of -1).");
+			P("The above constants may also be used anywhere variables are required.");
+			P("    sign, sign1, sign2, sign3, ... - variable that can only be +1 or -1.");
+			P("    integer - variable that may be any integer.");
+			P("    inf - infinity constant (a true constant).\n");
 			P("Absolute value notation \"|x|\" and \"+/-\" are understood.");
 			return true;
 		}
@@ -510,7 +512,7 @@ char	*cp;
 	}
 	/* default help text: */
 	P("This help command is provided as a quick reference.");
-	P("Type \"help all\" for a summary of the all commands.");
+	P("Type \"help all\" for a summary and syntax of the all commands.");
 	P("Type \"help usage\" for syntax of all commands.");
 	P("Type \"help expressions\" for help with entering expressions.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
