@@ -3,7 +3,7 @@
  *
  * These are very low level, so they don't do much, polynomials only.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -517,7 +517,7 @@ char	*cp;
 			return false;
 		}
 	}
-	if (extra_garbage(cp)) {
+	if (extra_characters(cp)) {
 		return false;
 	}
 	partial_flag = false;
@@ -601,6 +601,7 @@ char	*cp;
 		if (!prompt_var(&v))
 			return false;
 	}
+#if	!SILENT
 	singularity = false;
 	for (j = 1; j < *nps; j += 2) {
 		if (source[j].token.operatr == DIVIDE) {
@@ -614,6 +615,7 @@ char	*cp;
 	if (singularity) {
 		printf(_("Warning: Singularity detected, result of numerical integration may be wrong.\n"));
 	}
+#endif
 	my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 	if (!get_expr(tlhs, &n_tlhs)) {
 		return false;
@@ -635,11 +637,13 @@ char	*cp;
 	if ((n_tlhs + n_trhs + 3) > n_tokens) {
 		error_huge();
 	}
+#if	!SILENT
 	if (trap_flag) {
-		debug_string(0, _("Approximating the definite integral using the trapezoid method..."));
+		printf(_("Approximating definite integral using the trapezoid method (%d partitions)...\n"), iterations);
 	} else {
-		debug_string(0, _("Approximating the definite integral using Simpson's rule..."));
+		printf(_("Approximating definite integral using Simpson's rule (%d partitions)...\n"), iterations);
 	}
+#endif
 	subst_constants(source, nps);
 	simp_loop(source, nps);
 	for (j = 0; j < n_trhs; j++) {
@@ -743,6 +747,7 @@ char	*cp;
 			n1 += 2;
 		}
 
+		/* simplify and approximate the partial result quickly: */
 		approximate_roots = true;
 		elim_loop(dest, &n1);
 		ufactor(dest, &n1);
@@ -753,6 +758,7 @@ char	*cp;
 			error(_("Integration failed because result contains infinity or nan."));
 			return false;
 		}
+		/* detect an ever growing result: */
 		switch (j) {
 		case 0:
 			break;
@@ -796,13 +802,16 @@ char	*cp;
 	for (; k < n1; k++)
 		dest[k].level++;
 
+	/* simplify and approximate the result as before: */
 	approximate_roots = true;
 	elim_loop(dest, &n1);
 	ufactor(dest, &n1);
 	simp_divide(dest, &n1);
 	approximate_roots = false;
 
-	debug_string(0, _("Integration successful."));
+#if	!SILENT
+	printf(_("Integration successful.\n"));
+#endif
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
