@@ -2,13 +2,15 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
 
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
 
+#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
+
 static int	flist();
 static int	flist_recurse();
 
@@ -248,6 +250,9 @@ int	lang_code;	/* language code */
 		case -1:
 			cp = "%pi";
 			break;
+		case 0:
+			cp = "pi#";
+			break;
 		case 1:
 			cp = "M_PI";
 			break;
@@ -401,7 +406,7 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	len += 2;
 	cp = (char *) malloc(len);
 	if (cp == NULL) {
-		error(_("Can't malloc()."));
+		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
 	list_string_sub(lhs[n], n_lhs[n], cp, export_flag);
@@ -434,7 +439,7 @@ int		export_flag;
 	len++;
 	cp = (char *) malloc(len);
 	if (cp == NULL) {
-		error(_("Can't malloc()."));
+		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
 	list_string_sub(equation, n, cp, export_flag);
@@ -455,8 +460,6 @@ int		n;		/* length of expression */
 char		*string;	/* buffer or NULL pointer */
 int		export_flag;
 {
-#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
-
 	int	i, j, k, i1;
 	int	min1;
 	int	cur_level;
