@@ -1,7 +1,7 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -13,11 +13,11 @@ int	point_flag;		/* point to location of parse error if true */
 /*
  * Return true if character is a valid starting variable character.
  */
-static inline int
+int
 isvarchar(ch)
 int	ch;
 {
-	return(ch == '_' || ch == special_variable_character || isalpha(ch));
+	return(ch == '_' || (special_variable_character && ch == special_variable_character) || isalpha(ch));
 }
 
 /*
@@ -35,7 +35,7 @@ char	*cp;	/* pointer to the beginning of the equation character string */
 	}
 	if ((cp = parse_section(lhs[n], &n_lhs[n], cp)) != NULL) {
 		if ((cp = parse_section(rhs[n], &n_rhs[n], cp)) != NULL) {
-			if (extra_garbage(cp))
+			if (extra_characters(cp))
 				return NULL;
 			return cp;
 		}
@@ -60,8 +60,7 @@ char		*cp;		/* string to parse */
 	int		n = 0;		/* position in equation[] */
 	int		cur_level = 1;	/* current level of parentheses */
 	int		operand;	/* flip-flop between operand and operator */
-	char		*cp_start;
-	char		*cp1;
+	char		*cp_start, *cp1;
 	double		d;
 	int		abs_count = 0;
 	int		abs_array[10];
@@ -355,8 +354,7 @@ char	*cp;
 		error(_("Invalid variable."));
 		return(NULL);	/* variable name must start with a valid variable character */
 	}
-	cp1 = cp;
-	for (i = 0; *cp1;) {
+	for (cp1 = cp, i = 0; *cp1;) {
 		if (!isvarchar(*cp1)) {
 			break;
 		}
@@ -372,7 +370,6 @@ char	*cp;
 		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
 		vtmp = SIGN;
-		cp += strlen(buf);
 	} else {
 		if (strncasecmp(cp, "i#", 2) == 0) {
 			*vp = IMAGINARY;
@@ -382,21 +379,33 @@ char	*cp;
 			*vp = V_E;
 			return(cp + 2);
 		}
-		if ((*strcmpfunc)(buf, "pi") == 0) {
+		if (strncasecmp(cp, "pi#", 3) == 0) {
 			*vp = V_PI;
-			return(cp + 2);
+			return(cp + 3);
 		}
-		for (i = 0; *cp;) {
-			if (!isvarchar(*cp) && !isdigit(*cp)) {
+		for (cp1 = cp, i = 0; *cp1;) {
+			if (!isvarchar(*cp1) && !isdigit(*cp1)) {
 				break;
 			}
 			if (i >= MAX_VAR_LEN) {
 				error(_("Variable name too long."));
 				return(NULL);
 			}
-			buf[i++] = *cp++;
+			buf[i++] = *cp1++;
 		}
 		buf[i] = '\0';
+		if ((*strcmpfunc)(buf, "i") == 0) {
+			*vp = IMAGINARY;
+			return(cp1);
+		}
+		if ((*strcmpfunc)(buf, "e") == 0) {
+			*vp = V_E;
+			return(cp1);
+		}
+		if ((*strcmpfunc)(buf, "pi") == 0) {
+			*vp = V_PI;
+			return(cp1);
+		}
 		vtmp = 0;
 		for (i = 0; var_names[i]; i++) {
 			if ((*strcmpfunc)(buf, var_names[i]) == 0) {
@@ -422,9 +431,11 @@ char	*cp;
 			vtmp = i + VAR_OFFSET;
 			var_names[i+1] = NULL;
 		}
+		*vp = vtmp;
+		return cp1;
 	}
-	if (isdigit(*cp)) {
-		j = strtol(cp, &cp, 10);
+	if (isdigit(*cp1)) {
+		j = strtol(cp1, &cp1, 10);
 		if (j < 0 || j > MAX_SUBSCRIPT) {
 			error(_("Maximum subscript exceeded in special variable name."));
 			return(NULL);
@@ -435,7 +446,7 @@ char	*cp;
 		vtmp += ((long) (j + 1)) << VAR_SHIFT;
 	}
 	*vp = vtmp;
-	return cp;
+	return cp1;
 }
 
 /*
