@@ -1,12 +1,13 @@
 /*
- * Mathomatic simplify routines.
+ * Mathomatic simplifying routines.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
 
-#define	MAX_COMPARE_TERMS	400	/* Max no. of terms on same level that can be compared, increases stack usage. */
+#define	MAX_COMPARE_TERMS	500	/* Max no. of terms on same level that can be compared, increases stack usage. */
+					/* MAX_COMPARE_TERMS should be approximately (sqrt(DEFAULT_N_TOKENS) * 3) */
 
 static int	org_recurse();
 static int	const_recurse();
@@ -20,8 +21,12 @@ static int	order_recurse();
 simp_sub(n)
 int	n;	/* equation space number to simplify */
 {
+	if (n_lhs[n] <= 0)
+		return;
 	simp_loop(lhs[n], &n_lhs[n]);
-	simp_loop(rhs[n], &n_rhs[n]);
+	if (n_rhs[n]) {
+		simp_loop(rhs[n], &n_rhs[n]);
+	}
 }
 
 /*
@@ -76,7 +81,7 @@ int		*np;		/* pointer to length of equation side */
 
 /*
  * Simplify and approximate for the calculate command.
- * Includes imaginary number simplification.
+ * Includes complex number simplification.
  */
 calc_simp(equation, np)
 token_type	*equation;
@@ -85,6 +90,7 @@ int		*np;
 	approximate_roots = true;
 	subst_constants(equation, np);
 	simp_side(equation, np);
+	factorv(equation, np, IMAGINARY);
 	ufactor(equation, np);
 	factorv(equation, np, IMAGINARY);
 	ufactor(equation, np);
@@ -218,7 +224,7 @@ do_quick:
 	}
 	partial_flag = false;
 	simpb_side(equation, np, true, 1);
-	debug_string(1, "Simplified result before applying polynomial operations:");
+	debug_string(1, "Simplify result before applying polynomial operations:");
 	side_debug(1, equation, *np);
 	for (flag = false;;) {
 		/* divide top and bottom of fractions by any polynomial GCD found */
@@ -258,6 +264,8 @@ do_quick:
 	}
 	poly_factor(equation, np);
 	simpb_side(equation, np, true, 2);
+	debug_string(1, "Result of complete simplify:");
+	side_debug(1, equation, *np);
 }
 
 /*
@@ -409,8 +417,6 @@ int		*np;		/* pointer to length of equation side */
 	int	i;
 	int	rv = false;
 
-	if (*np <= 1)	/* no need to simplify a constant or a variable */
-		return rv;
 	do {
 		do {
 			do {
@@ -495,11 +501,11 @@ token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 {
 	if (*np <= 0 || (*np & 1) != 1) {
-		error(_("Organize called with bad expression size!"));
+		error("Internal error: Organize called with bad expression size.");
 		longjmp(jmp_save, 13);
 	}
 	if (*np > n_tokens) {
-		error(_("Array overflow detected!  Expression larger than \"n_tokens\"."));
+		error("Internal error: Expression array overflow detected.");
 		longjmp(jmp_save, 13);
 	}
 	org_recurse(equation, np, 0, 1, NULL);
